"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectErrorCommand = void 0;
var _jqlAst = require("@atlassianlabs/jql-ast");
var _prosemirrorState = require("prosemirror-state");
var _getDocumentPosition = _interopRequireDefault(require("../../plugins/common/get-document-position"));
var _jqlAst2 = require("../../plugins/jql-ast");
/**
 * A Prosemirror Command that will select the first JQL error in the query if present.
 */
var selectErrorCommand = function selectErrorCommand(state, _, view) {
  var jast = (0, _jqlAst2.getJastFromState)(state);
  var error = jast.errors.length > 0 ? jast.errors[0] : null;
  if (!(error instanceof _jqlAst.JQLSyntaxError) || !view) {
    return false;
  }
  var transaction = state.tr;
  var documentFrom = (0, _getDocumentPosition.default)(transaction.doc, error.start);
  var documentTo = (0, _getDocumentPosition.default)(transaction.doc, error.stop);
  var selection = _prosemirrorState.TextSelection.create(transaction.doc, documentFrom, documentTo);
  transaction.setSelection(selection);
  transaction.scrollIntoView();
  view.focus();
  view.dispatch(transaction);
  return true;
};
exports.selectErrorCommand = selectErrorCommand;