"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _typeof = require("@babel/runtime/helpers/typeof");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _react = _interopRequireWildcard(require("react"));
var _jqlAst = require("@atlassianlabs/jql-ast");
var _deburr = _interopRequireDefault(require("lodash/deburr"));
var _noop = _interopRequireDefault(require("lodash/noop"));
var _icon = _interopRequireDefault(require("@atlaskit/icon"));
var _panel = _interopRequireDefault(require("@atlaskit/icon/glyph/editor/panel"));
var _colors = require("@atlaskit/theme/colors");
var _tooltip = _interopRequireDefault(require("@atlaskit/tooltip"));
var _state = require("../../../../state");
var _glyphs = require("./glyphs");
var _messages = require("./messages");
var _styled = require("./styled");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
var ResizedIcon = function ResizedIcon(_ref) {
  var glyph = _ref.glyph;
  return /*#__PURE__*/_react.default.createElement(_icon.default, {
    glyph: glyph,
    size: "small",
    label: "",
    testId: "jql-editor-field-type-icon"
  });
};

/**
 * List of types with supported icons is derived from GIN
 * Custom SVGs are provisional while we work on making field type icons consistent across Atlassian
 */
var getFieldTypeIcon = function getFieldTypeIcon(type) {
  switch (type) {
    case 'Checkboxes':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.checkboxGlyph
      });
    case 'Date':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.dateGlyph
      });
    case 'Dropdown':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.dropdownGlyph
      });
    case 'Labels':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.labelGlyph
      });
    case 'Number':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.numberGlyph
      });
    case 'Paragraph':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.paragraphGlyph
      });
    case 'People':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.peopleGlyph
      });
    case 'Short text':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.shortTextGlyph
      });
    case 'Time stamp':
      return /*#__PURE__*/_react.default.createElement(ResizedIcon, {
        glyph: _glyphs.timeStampGlyph
      });
    default:
      return null;
  }
};
var getHighlightPosition = function getHighlightPosition(name, matchedText) {
  var normalizedName = (0, _deburr.default)(name).toLowerCase();
  var normalizedMatchedText = (0, _deburr.default)((0, _jqlAst.normaliseJqlString)(matchedText)).toLowerCase();
  var index = normalizedName.indexOf(normalizedMatchedText);
  if (index === -1) {
    return null;
  }
  return [index, index + normalizedMatchedText.length];
};
var getDeprecatedTooltipMessage = function getDeprecatedTooltipMessage(deprecatedSearcherKey) {
  switch (deprecatedSearcherKey) {
    case 'com.pyxis.greenhopper.jira:gh-epic-link-searcher':
    case 'com.atlassian.jpo:jpo-custom-field-parent-searcher':
      return _messages.messages.deprecatedFieldTooltipParentReplacementMessage;
    default:
      return _messages.messages.deprecatedFieldTooltipDefaultMessage;
  }
};
var AutocompleteOption = /*#__PURE__*/(0, _react.forwardRef)(function (props, ref) {
  var _props$option = props.option,
    id = _props$option.id,
    name = _props$option.name,
    fieldType = _props$option.fieldType,
    matchedText = _props$option.matchedText,
    _props$option$isDepre = _props$option.isDeprecated,
    isDeprecated = _props$option$isDepre === void 0 ? false : _props$option$isDepre,
    deprecatedSearcherKey = _props$option.deprecatedSearcherKey,
    isSelected = props.isSelected,
    onClick = props.onClick,
    onMouseMove = props.onMouseMove;
  var _useState = (0, _react.useState)(false),
    _useState2 = (0, _slicedToArray2.default)(_useState, 2),
    overflows = _useState2[0],
    setOverflows = _useState2[1];
  var _useIntl = (0, _state.useIntl)(),
    _useIntl2 = (0, _slicedToArray2.default)(_useIntl, 1),
    formatMessage = _useIntl2[0].formatMessage;
  var onNameRef = (0, _react.useCallback)(function (node) {
    if (node != null) {
      setOverflows(node.offsetWidth < node.scrollWidth);
    }
  }, []);

  // TODO: verify type is i18n'd after integrating with GraphQL API

  var fieldTypeIcon = fieldType ? getFieldTypeIcon(fieldType) : null;
  var optionName = (0, _react.useMemo)(function () {
    var highlightPosition = getHighlightPosition(name, matchedText);
    return highlightPosition ? /*#__PURE__*/_react.default.createElement(_styled.OptionName, {
      ref: onNameRef,
      "aria-label": name
    }, name.substring(0, highlightPosition[0]), /*#__PURE__*/_react.default.createElement(_styled.OptionHighlight, null, name.substring.apply(name, (0, _toConsumableArray2.default)(highlightPosition))), name.substring(highlightPosition[1])) : /*#__PURE__*/_react.default.createElement(_styled.OptionName, {
      ref: onNameRef
    }, name);
  }, [onNameRef, name, matchedText]);
  var deprecatedTooltipContent = /*#__PURE__*/_react.default.createElement(_styled.TooltipContent, null, formatMessage(getDeprecatedTooltipMessage(deprecatedSearcherKey), {
    b: function b(text) {
      return /*#__PURE__*/_react.default.createElement("b", null, text);
    },
    received: name,
    parentReplacement: 'Parent'
  }));
  var ListItem = /*#__PURE__*/_react.default.createElement(_styled.OptionListItem, (0, _extends2.default)({
    id: id,
    "data-testid": "jql-editor-autocomplete-option",
    role: "option",
    isSelected: isSelected,
    isDeprecated: isDeprecated,
    onClick: isDeprecated ? _noop.default : onClick,
    onMouseMove: isDeprecated ? _noop.default : onMouseMove
  }, ref && {
    ref: ref
  }, {
    "aria-selected": isSelected
  }), isDeprecated ? /*#__PURE__*/_react.default.createElement(_styled.DeprecatedOptionContainer, null, optionName, /*#__PURE__*/_react.default.createElement(_tooltip.default, {
    content: deprecatedTooltipContent,
    position: 'right'
  }, /*#__PURE__*/_react.default.createElement(_panel.default, {
    testId: "jql-editor-deprecated-icon",
    label: "",
    primaryColor: "var(--ds-icon, ".concat(_colors.N400, ")")
  }))) : optionName, fieldType && /*#__PURE__*/_react.default.createElement(_styled.FieldType, null, fieldTypeIcon && /*#__PURE__*/_react.default.createElement(_styled.FieldTypeIcon, null, fieldTypeIcon), fieldType));
  var tooltipContent = /*#__PURE__*/_react.default.createElement(_styled.TooltipContent, null, name);
  if (overflows) {
    return /*#__PURE__*/_react.default.createElement(_tooltip.default, {
      content: tooltipContent,
      position: 'right'
    }, ListItem);
  }
  return ListItem;
});
var _default = AutocompleteOption;
exports.default = _default;