"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultEditorState = exports.configurePlugins = exports.clipboardTextSerializer = exports.clipboardTextParser = exports.JQLEditorSchema = void 0;
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _mapValues = _interopRequireDefault(require("lodash/mapValues"));
var _prosemirrorCommands = require("prosemirror-commands");
var _prosemirrorHistory = require("prosemirror-history");
var _prosemirrorKeymap = require("prosemirror-keymap");
var _prosemirrorModel = require("prosemirror-model");
var _prosemirrorState = require("prosemirror-state");
var _plugins = require("../plugins");
var _constants = require("../plugins/rich-inline-nodes/constants");
var _nodes = require("../plugins/rich-inline-nodes/nodes");
var _createNodeSpec = require("../plugins/rich-inline-nodes/util/create-node-spec");
var _documentText = require("../utils/document-text");
var _splitTextByNewLine = require("../utils/split-text-by-new-line");
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
var createMarkWithStyle = function createMarkWithStyle(tokenType) {
  var mark = document.createElement('span');
  mark.className = "mark-token-".concat(tokenType);
  mark.setAttribute('data-token-type', tokenType);
  mark.setAttribute('spellcheck', 'false');
  return mark;
};
var JQLEditorSchema = new _prosemirrorModel.Schema({
  nodes: _objectSpread({
    doc: {
      content: 'paragraph+'
    },
    paragraph: {
      content: "(text|".concat(_constants.RICH_INLINE_NODE, ")*"),
      toDOM: function toDOM() {
        return ['p', {
          spellcheck: 'false'
        }, 0];
      },
      parseDOM: [{
        tag: 'p'
      }]
    },
    text: {}
  }, (0, _mapValues.default)(_nodes.richInlineNodes, function (spec, name) {
    return (0, _createNodeSpec.createNodeSpec)(name, spec.attrs);
  })),
  marks: {
    token: {
      toDOM: function toDOM(_ref) {
        var attrs = _ref.attrs;
        return createMarkWithStyle(attrs.tokenType);
      },
      attrs: {
        tokenType: {}
      }
    },
    cursor: {
      toDOM: function toDOM() {
        return document.createElement('span');
      }
    }
  }
});
exports.JQLEditorSchema = JQLEditorSchema;
var domParser = _prosemirrorModel.DOMParser.fromSchema(JQLEditorSchema);

/**
 * Emulate the behaviour of the default https://prosemirror.net/docs/ref/#view.EditorProps.clipboardTextSerializer but
 * preserves consecutive empty block nodes.
 */
var clipboardTextSerializer = function clipboardTextSerializer(slice) {
  return (0, _documentText.getFragmentText)(slice.content, 0, slice.content.size);
};

/**
 * Emulate the behaviour of the default https://prosemirror.net/docs/ref/#view.EditorProps.clipboardTextParser but
 * preserves consecutive empty lines.
 */
exports.clipboardTextSerializer = clipboardTextSerializer;
var clipboardTextParser = function clipboardTextParser(text, $context) {
  var dom = document.createElement('div');
  // Split each line of text and wrap each in a p tag.
  (0, _splitTextByNewLine.splitTextByNewLine)(text).forEach(function (block) {
    dom.appendChild(document.createElement('p')).textContent = block;
  });
  return domParser.parseSlice(dom, {
    context: $context,
    preserveWhitespace: true
  });
};
exports.clipboardTextParser = clipboardTextParser;
var defaultEditorState = _prosemirrorState.EditorState.create({
  schema: JQLEditorSchema
});

// @types/prosemirror-commands@1.0.1 does not export Command and Keymap types
// TODO: update to Command<JQLEditorSchemaType> and Keymap<JQLEditorSchemaType> if types are bumped to a newer version
exports.defaultEditorState = defaultEditorState;
var configurePlugins = function configurePlugins(editorState, onSearchCommand, intlRef, mainId, portalActions, enableRichInlineNodes) {
  return editorState.reconfigure({
    plugins: [(0, _prosemirrorHistory.history)(),
    // Other plugins rely on having AST as part of the state during a transaction, this plugin should be kept first
    (0, _plugins.jqlAstPlugin)(intlRef), (0, _plugins.jqlSyntaxHighlightingPlugin)()].concat((0, _toConsumableArray2.default)(portalActions !== undefined ? [(0, _plugins.autocompletePlugin)(portalActions, enableRichInlineNodes)].concat((0, _toConsumableArray2.default)(enableRichInlineNodes ? [(0, _plugins.richInlineNodesPlugin)(portalActions)] : [])) : []), [(0, _plugins.validationTooltipPlugin)(mainId),
    // Keeping these at the bottom allows plugins to define custom key bindings to override default behavior
    (0, _prosemirrorKeymap.keymap)({
      'Mod-z': _prosemirrorHistory.undo,
      'Mod-Shift-z': _prosemirrorHistory.redo,
      'Mod-y': _prosemirrorHistory.redo,
      // Mimic default Enter behavior in PM's base keymap, allowing to insert new lines even when autocomplete is open
      'Shift-Enter': _prosemirrorCommands.baseKeymap.Enter
    })], (0, _toConsumableArray2.default)(!onSearchCommand ? [] : [keymapNoRepeat({
      'Mod-Enter': onSearchCommand,
      Enter: onSearchCommand
    })]), [(0, _prosemirrorKeymap.keymap)(_prosemirrorCommands.baseKeymap)])
  });
};
exports.configurePlugins = configurePlugins;
var noopCommand = function noopCommand() {
  return true;
};
function keymapNoRepeat(bindings) {
  var handler = (0, _prosemirrorKeymap.keydownHandler)(bindings);
  var proxyBindings = (0, _mapValues.default)(bindings, function () {
    return noopCommand;
  });
  var proxyHandler = (0, _prosemirrorKeymap.keydownHandler)(proxyBindings);
  return new _prosemirrorState.Plugin({
    props: {
      handleKeyDown: function handleKeyDown(view, event) {
        if (event.repeat) {
          return proxyHandler(view, event);
        }
        return handler(view, event);
      }
    }
  });
}