"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ValidQueryVisitor = void 0;
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));
var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));
var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));
var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _jqlAst = require("@atlassianlabs/jql-ast");
function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = (0, _getPrototypeOf2.default)(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = (0, _getPrototypeOf2.default)(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return (0, _possibleConstructorReturn2.default)(this, result); }; }
function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }
/**
 * Given an AST with parse errors, this visitor returns a valid query that is equivalent for hydration purposes
 * (i.e. just fields, operators and values). Resulting query is generated on a best-effort basis and depends on
 * a successful parser error recovery.
 *
 * Example 1:
 * - Original query: "assignee in (abc-123-def"
 * - Equivalent query: "assignee in (abc-123-def)"
 *
 * Example 2:
 * - Original query: "project = EM and status in (Done, currentUser(), EMPTY) and reporter in"
 * - Equivalent query: "project = EM and status in (Done)"
 */
var ValidQueryVisitor = /*#__PURE__*/function (_AbstractJastVisitor) {
  (0, _inherits2.default)(ValidQueryVisitor, _AbstractJastVisitor);
  var _super = _createSuper(ValidQueryVisitor);
  function ValidQueryVisitor() {
    var _this;
    (0, _classCallCheck2.default)(this, ValidQueryVisitor);
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    _this = _super.call.apply(_super, [this].concat(args));
    (0, _defineProperty2.default)((0, _assertThisInitialized2.default)(_this), "visitQuery", function (query) {
      if (!query.where) {
        return '';
      }
      return query.where.accept((0, _assertThisInitialized2.default)(_this));
    });
    (0, _defineProperty2.default)((0, _assertThisInitialized2.default)(_this), "visitCompoundClause", function (compoundClause) {
      return compoundClause.clauses.map(function (clause) {
        return clause.accept((0, _assertThisInitialized2.default)(_this));
      }).filter(function (value) {
        return !!value;
      }).join(' and ');
    });
    (0, _defineProperty2.default)((0, _assertThisInitialized2.default)(_this), "visitTerminalClause", function (terminalClause) {
      var field = terminalClause.field,
        operator = terminalClause.operator,
        operand = terminalClause.operand;
      if (!operator || !operand) {
        return '';
      }
      var operandValue = operand.accept((0, _assertThisInitialized2.default)(_this));
      if (!operandValue) {
        return '';
      }
      return "".concat(field.text, " ").concat(operator.value, " ").concat(operandValue);
    });
    (0, _defineProperty2.default)((0, _assertThisInitialized2.default)(_this), "visitNotClause", function (notClause) {
      return notClause.clause.accept((0, _assertThisInitialized2.default)(_this));
    });
    (0, _defineProperty2.default)((0, _assertThisInitialized2.default)(_this), "visitValueOperand", function (valueOperand) {
      return valueOperand.text;
    });
    (0, _defineProperty2.default)((0, _assertThisInitialized2.default)(_this), "visitListOperand", function (listOperand) {
      return "(".concat(listOperand.values.map(function (value) {
        return value.accept((0, _assertThisInitialized2.default)(_this));
      }).filter(function (value) {
        return !!value;
      }).join(', '), ")");
    });
    return _this;
  }
  (0, _createClass2.default)(ValidQueryVisitor, [{
    key: "defaultResult",
    value: function defaultResult() {
      return '';
    }
  }]);
  return ValidQueryVisitor;
}(_jqlAst.AbstractJastVisitor);
exports.ValidQueryVisitor = ValidQueryVisitor;