"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _typeof = require("@babel/runtime/helpers/typeof");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useFormattedWarningMessage = exports.WarningMessages = void 0;
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _react = _interopRequireWildcard(require("react"));
var _reactMagneticDi = require("react-magnetic-di");
var _form = require("@atlaskit/form");
var _warning = _interopRequireDefault(require("@atlaskit/icon/glyph/warning"));
var colors = _interopRequireWildcard(require("@atlaskit/theme/colors"));
var _useEditorViewHasWarnings = require("../../../../hooks/use-editor-view-has-warnings");
var _state = require("../../../../state");
var _format = require("../format");
var _messages = require("./messages");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
var softDeprecateEpicParentDocsLink = 'https://support.atlassian.com/jira-software-cloud/docs/upcoming-changes-epic-link-replaced-with-parent/';
var getEpicLinkDeprecationTerm = function getEpicLinkDeprecationTerm(hydratedDeprecations) {
  var epicLinkWarning = hydratedDeprecations.filter(function (warning) {
    return warning.deprecatedSearcherKey === 'com.pyxis.greenhopper.jira:gh-epic-link-searcher';
  });
  return epicLinkWarning.length > 0 ? epicLinkWarning[0].id : null;
};
var getParentLinkDeprecationTerm = function getParentLinkDeprecationTerm(hydratedDeprecations) {
  var parentLinkWarning = hydratedDeprecations.filter(function (warning) {
    return warning.deprecatedSearcherKey === 'com.atlassian.jpo:jpo-custom-field-parent-searcher';
  });
  return parentLinkWarning.length > 0 ? parentLinkWarning[0].id : null;
};
var useFormattedWarningMessage = function useFormattedWarningMessage() {
  var _useHydratedDeprecati = (0, _state.useHydratedDeprecations)(),
    _useHydratedDeprecati2 = (0, _slicedToArray2.default)(_useHydratedDeprecati, 1),
    hydratedDeprecations = _useHydratedDeprecati2[0];
  var _useExternalMessages = (0, _state.useExternalMessages)(),
    _useExternalMessages2 = (0, _slicedToArray2.default)(_useExternalMessages, 1),
    externalWarnings = _useExternalMessages2[0].warnings;
  var hasWarnings = (0, _useEditorViewHasWarnings.useEditorViewHasWarnings)();
  var _useIntl = (0, _state.useIntl)(),
    _useIntl2 = (0, _slicedToArray2.default)(_useIntl, 1),
    formatMessage = _useIntl2[0].formatMessage;
  var epicLinkDeprecationTerm = getEpicLinkDeprecationTerm(hydratedDeprecations);
  var parentLinkDeprecationTerm = getParentLinkDeprecationTerm(hydratedDeprecations);
  var formatWarnings = (0, _react.useCallback)(function (message) {
    var combinedMessages = [{
      type: 'warning',
      message: message
    }].concat((0, _toConsumableArray2.default)(externalWarnings));
    return /*#__PURE__*/_react.default.createElement(_format.FormatMessages, {
      messages: combinedMessages
    });
  }, [externalWarnings]);
  if (!hasWarnings) {
    return null;
  } else if (!hydratedDeprecations.length && externalWarnings.length) {
    return /*#__PURE__*/_react.default.createElement(_format.FormatMessages, {
      messages: externalWarnings
    });
  } else if (epicLinkDeprecationTerm == null && parentLinkDeprecationTerm == null) {
    var _hydratedDeprecations;
    return formatWarnings(formatMessage(_messages.messages.defaultWarning, {
      deprecatedField: (_hydratedDeprecations = hydratedDeprecations[0]) === null || _hydratedDeprecations === void 0 ? void 0 : _hydratedDeprecations.id
    }));
  } else if (epicLinkDeprecationTerm && parentLinkDeprecationTerm) {
    return formatWarnings(formatMessage(_messages.messages.deprecatedBothParentReplacementMessage, {
      link: function link(chunks) {
        return /*#__PURE__*/_react.default.createElement("a", {
          href: softDeprecateEpicParentDocsLink,
          target: "_blank",
          rel: "noopener noreferrer",
          onClick: function onClick(e) {
            return e.stopPropagation();
          }
        }, chunks);
      },
      receivedFirst: epicLinkDeprecationTerm,
      receivedSecond: parentLinkDeprecationTerm,
      parentReplacement: 'Parent'
    }));
  } else if (epicLinkDeprecationTerm || parentLinkDeprecationTerm) {
    return formatWarnings(formatMessage(_messages.messages.deprecatedParentReplacementMessage, {
      link: function link(chunks) {
        return /*#__PURE__*/_react.default.createElement("a", {
          href: softDeprecateEpicParentDocsLink,
          target: "_blank",
          rel: "noopener noreferrer",
          onClick: function onClick(e) {
            return e.stopPropagation();
          }
        }, chunks);
      },
      deprecatedField: epicLinkDeprecationTerm ? epicLinkDeprecationTerm : parentLinkDeprecationTerm,
      parentReplacement: 'Parent'
    }));
  }
};
exports.useFormattedWarningMessage = useFormattedWarningMessage;
var WarningMessages = function WarningMessages() {
  var warningMessage = useFormattedWarningMessage();
  return warningMessage != null ? /*#__PURE__*/_react.default.createElement(_format.MessageContainer, null, /*#__PURE__*/_react.default.createElement(_form.HelperMessage, {
    testId: "jql-editor-warning-message"
  }, /*#__PURE__*/_react.default.createElement(_warning.default, {
    label: "",
    primaryColor: "var(--ds-icon-warning, ".concat(colors.Y400, ")"),
    size: "small"
  }), warningMessage)) : null;
};
exports.WarningMessages = WarningMessages;