import _defineProperty from "@babel/runtime/helpers/defineProperty";
import { AbstractJastVisitor } from '@atlassianlabs/jql-ast';
import { JQLEditorSchema } from '../../../schema';
import getDocumentPosition from '../../common/get-document-position';
import { getJastFromState } from '../../jql-ast';
import { RICH_INLINE_NODE } from '../constants';
export const replaceRichInlineNodes = (editorState, hydratedValues) => {
  const ast = getJastFromState(editorState);
  const transaction = editorState.tr;
  // Do not allow users to revert this transaction with undo command so they can't land in an unhydrated state
  transaction.setMeta('addToHistory', false);
  Object.entries(hydratedValues).forEach(([fieldName, values]) => {
    values.forEach(value => {
      if (value.type === 'user') {
        const astNodes = getValueNodes(ast, fieldName, value.id);
        astNodes.forEach(astNode => {
          if (astNode.position) {
            const [from, to] = astNode.position;
            const documentFrom = getDocumentPosition(transaction.doc, from);
            if (!isRichInlineNode(transaction.doc, documentFrom)) {
              const documentTo = getDocumentPosition(transaction.doc, to);
              const node = getRichInlineNode(fieldName, value, astNode.text);
              transaction.replaceWith(documentFrom, documentTo, node);
            }
          }
        });
      }
    });
  });
  return transaction;
};
const getRichInlineNode = (fieldName, value, text) => {
  switch (value.type) {
    case 'user':
      {
        const textContent = JQLEditorSchema.text(text);
        return JQLEditorSchema.nodes.user.create({
          ...value,
          fieldName
        }, textContent);
      }
    default:
      {
        throw new Error(`Unsupported hydrated value type ${value.type}`);
      }
  }
};
const isRichInlineNode = (doc, position) => {
  var _doc$resolve$nodeAfte;
  return ((_doc$resolve$nodeAfte = doc.resolve(position).nodeAfter) === null || _doc$resolve$nodeAfte === void 0 ? void 0 : _doc$resolve$nodeAfte.type.spec.group) === RICH_INLINE_NODE;
};
const getValueNodes = (ast, field, value) => {
  if (!ast.query) {
    return [];
  }
  return ast.query.accept(new FindValuesVisitor(field, value));
};
class FindValuesVisitor extends AbstractJastVisitor {
  constructor(field, value) {
    super();
    _defineProperty(this, "visitQuery", query => {
      if (query.where === undefined) {
        return [];
      }
      return query.where.accept(this);
    });
    _defineProperty(this, "visitCompoundClause", compoundClause => {
      return compoundClause.clauses.reduce((operands, compoundClause) => {
        return [...operands, ...compoundClause.accept(this)];
      }, []);
    });
    _defineProperty(this, "visitTerminalClause", terminalClause => {
      if (terminalClause.field.value !== this.field) {
        return [];
      }
      if (terminalClause.operand === undefined) {
        return [];
      }
      return terminalClause.operand.accept(this);
    });
    _defineProperty(this, "visitNotClause", notClause => {
      return notClause.clause.accept(this);
    });
    _defineProperty(this, "visitValueOperand", valueOperand => {
      if (valueOperand.value !== this.value) {
        return [];
      }
      return [valueOperand];
    });
    _defineProperty(this, "visitListOperand", listOperand => {
      return listOperand.values.reduce((values, operand) => {
        return [...values, ...operand.accept(this)];
      }, []);
    });
    this.field = field;
    this.value = value;
  }
  aggregateResult(aggregate, nextResult) {
    return [...aggregate, ...nextResult];
  }
  defaultResult() {
    return [];
  }
}