import React from 'react';
import Button from '@atlaskit/button';
import MediaServicesActualSizeIcon from '@atlaskit/icon/glyph/media-services/actual-size';
import MediaServicesFitToPageIcon from '@atlaskit/icon/glyph/media-services/fit-to-page';
import { N50, N500 } from '@atlaskit/theme/colors';
import { ExpandToggleContainer } from './styled';

// Atlaskit doesn't provide a circle variation of the expand/collapse icons so we have to implement our own
const ExpandCircleIcon = ({
  isDisabled
}) => /*#__PURE__*/React.createElement(MediaServicesActualSizeIcon, {
  label: '',
  size: 'small',
  primaryColor: isDisabled ? `var(--ds-icon-disabled, ${N50})` : `var(--ds-icon, ${N500})`
});
const CollapseCircleIcon = ({
  isDisabled
}) => /*#__PURE__*/React.createElement(MediaServicesFitToPageIcon, {
  label: '',
  size: 'small',
  primaryColor: isDisabled ? `var(--ds-icon-disabled, ${N50})` : `var(--ds-icon, ${N500})`
});
export const BaseExpandToggle = ({
  expanded,
  editorId,
  isDisabled,
  label,
  onClick
}) => {
  let Icon;
  if (expanded) {
    Icon = CollapseCircleIcon;
  } else {
    Icon = ExpandCircleIcon;
  }
  return /*#__PURE__*/React.createElement(ExpandToggleContainer, null, /*#__PURE__*/React.createElement(Button, {
    appearance: 'subtle',
    "aria-expanded": expanded,
    "aria-controls": editorId,
    "aria-label": label,
    isDisabled: isDisabled,
    spacing: 'none',
    onClick: onClick,
    iconBefore: /*#__PURE__*/React.createElement(Icon, {
      isDisabled: isDisabled
    })
  }));
};