import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React, { useEffect } from 'react';
import { JQLSyntaxError } from '@atlassianlabs/jql-ast';
import { di } from 'react-magnetic-di';
import { ErrorMessage } from '@atlaskit/form';
import { JQL_EDITOR_INPUT_ID } from '../../../../common/constants';
import { commonMessages } from '../../../../common/messages';
import { useEditorViewIsInvalid } from '../../../../hooks/use-editor-view-is-invalid';
import { useExternalMessages, useIntl, useJqlError, useScopedId, useStoreActions } from '../../../../state';
import { FormatMessages, MessageContainer } from '../format';
import { messages } from './messages';
var useFormattedErrorMessage = function useFormattedErrorMessage() {
  var _useIntl = useIntl(),
    _useIntl2 = _slicedToArray(_useIntl, 1),
    formatMessage = _useIntl2[0].formatMessage;
  var _useJqlError = useJqlError(),
    _useJqlError2 = _slicedToArray(_useJqlError, 1),
    jqlError = _useJqlError2[0];
  var _useExternalMessages = useExternalMessages(),
    _useExternalMessages2 = _slicedToArray(_useExternalMessages, 1),
    externalErrors = _useExternalMessages2[0].errors;
  var _useStoreActions = useStoreActions(),
    _useStoreActions2 = _slicedToArray(_useStoreActions, 2),
    externalErrorMessageViewed = _useStoreActions2[1].externalErrorMessageViewed;
  var editorViewIsInvalid = useEditorViewIsInvalid();
  useEffect(function () {
    // Emit a UI event whenever external errors has changed
    externalErrorMessageViewed();
  }, [externalErrorMessageViewed, externalErrors]);
  if (!editorViewIsInvalid) {
    return null;
  }

  // Prioritise client over external errors
  if (jqlError instanceof JQLSyntaxError) {
    return "".concat(jqlError.message, " ").concat(formatMessage(messages.jqlErrorPosition, {
      lineNumber: jqlError.line,
      charPosition: jqlError.charPositionInLine + 1
    }));
  } else if (externalErrors.length > 0) {
    return /*#__PURE__*/React.createElement(FormatMessages, {
      messages: externalErrors
    });
  } else if (jqlError !== null) {
    return formatMessage(commonMessages.unknownError);
  }
  return null;
};
export var ErrorMessages = function ErrorMessages() {
  var _useScopedId = useScopedId(JQL_EDITOR_INPUT_ID),
    _useScopedId2 = _slicedToArray(_useScopedId, 1),
    editorId = _useScopedId2[0];
  var errorMessage = useFormattedErrorMessage();
  return errorMessage != null ? /*#__PURE__*/React.createElement(MessageContainer, null, /*#__PURE__*/React.createElement(ErrorMessage, {
    testId: "jql-editor-validation"
  }, /*#__PURE__*/React.createElement("span", {
    role: "alert",
    "aria-describedby": editorId
  }, errorMessage))) : null;
};