/// <reference types="react" />
export type EditorTheme = {
    /**
     * The maximum number of rows that are visible in the default editor view.
     */
    defaultMaxRows: number;
    /**
     * `true` if the editor view is in the expanded state.
     */
    expanded: boolean;
    /**
     * The number of rows that are visible when the editor view is expanded.
     */
    expandedRows: number;
    /**
     * Toggle the editor view expanded state.
     */
    toggleExpanded: () => void;
    /**
     * `true` shows search button, other search related elements
     * `false` to act as a text field
     */
    isSearch: boolean;
    /**
     * `false` matches AK's default field styling
     * `true` matches AK's compact field styling, generally used for search purposes.
     */
    isCompact: boolean;
};
export declare const EditorThemeContext: import("react").Context<EditorTheme>;
/**
 * Hook to manage the theming state of the editor.
 */
export declare const useEditorTheme: ({ isSearch, isCompact, }: {
    isSearch?: boolean | undefined;
    isCompact?: boolean | undefined;
}) => EditorTheme;
export declare const useEditorThemeContext: () => EditorTheme;
