import { MutableRefObject } from 'react';
import { ResolvedPos, Schema, Slice } from 'prosemirror-model';
import { EditorState, Transaction } from 'prosemirror-state';
import { EditorView } from 'prosemirror-view';
import { IntlShape } from 'react-intl-next';
import { richInlineNodes } from '../plugins/rich-inline-nodes/nodes';
import { PortalActions } from '../ui/jql-editor-portal-provider/types';
type Nodes = 'doc' | 'paragraph' | 'text' | keyof typeof richInlineNodes;
type Marks = 'token' | 'cursor';
export declare const JQLEditorSchema: Schema<Nodes, Marks>;
/**
 * Emulate the behaviour of the default https://prosemirror.net/docs/ref/#view.EditorProps.clipboardTextSerializer but
 * preserves consecutive empty block nodes.
 */
export declare const clipboardTextSerializer: (slice: Slice) => string;
/**
 * Emulate the behaviour of the default https://prosemirror.net/docs/ref/#view.EditorProps.clipboardTextParser but
 * preserves consecutive empty lines.
 */
export declare const clipboardTextParser: (text: string, $context: ResolvedPos) => Slice;
export declare const defaultEditorState: EditorState;
export type JQLEditorSchemaType = typeof JQLEditorSchema;
export type JQLEditorCommand = (state: EditorState, dispatch?: (tr: Transaction) => void, view?: EditorView) => boolean;
export type JQLEditorKeymap = {
    [key: string]: JQLEditorCommand;
};
export declare const configurePlugins: (editorState: EditorState, onSearchCommand: JQLEditorCommand | undefined, intlRef: MutableRefObject<IntlShape>, mainId: string, portalActions: PortalActions | void, enableRichInlineNodes: boolean) => EditorState;
export {};
